@@ -7,20 +7,8 @@
  *         Jorn Baayen <jorn@openedhand.com>
  *         Ludovic Ferrandis <ludovic.ferrandis@intel.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -80,7 +68,7 @@ typedef struct _GUPnPContextManagerPrivate GUPnPContextManagerPrivate;
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GUPnPContextManager,
                                      gupnp_context_manager,
-                                     G_TYPE_OBJECT);
+                                     G_TYPE_OBJECT)
 
 enum {
         PROP_0,
@@ -131,7 +119,7 @@ on_context_available (GUPnPContextManager    *manager,
         if (!gupnp_white_list_is_empty (white_list) &&
             gupnp_white_list_get_enabled (white_list) &&
             !gupnp_white_list_check_context (white_list, context)) {
-                /* If the conext doesn't match, block the notification
+                /* If the context doesn't match, block the notification
                  * and disable the context */
                 g_signal_stop_emission_by_name (manager, "context-available");
 
@@ -513,7 +501,7 @@ gupnp_context_manager_class_init (GUPnPContextManagerClass *klass)
          * for any or %G_SOCKET_FAMILY_IPV4 for IPv4 contexts or
          * %G_SOCKET_FAMILY_IPV6 for IPv6 contexts
          *
-         * Since: 1.1.0
+         * Since: 1.2.0
          **/
         g_object_class_install_property
                 (object_class,
@@ -533,7 +521,7 @@ gupnp_context_manager_class_init (GUPnPContextManagerClass *klass)
          * The UDA version the contexts will support. Use %GSSDP_UDA_VERSION_UNSPECIFIED
          * for using the default UDA version.
          *
-         * Since: 1.1.2
+         * Since: 1.2.0
          **/
         g_object_class_install_property
                 (object_class,
@@ -619,7 +607,7 @@ gupnp_context_manager_class_init (GUPnPContextManagerClass *klass)
  *
  * Returns: (transfer full): A new #GUPnPContextManager object.
  *
- * Since: 0.17.2
+ * Since: 0.18.0
  **/
 GUPnPContextManager *
 gupnp_context_manager_create (guint port)
@@ -643,7 +631,7 @@ gupnp_context_manager_create (guint port)
  *
  * Returns: (transfer full): A new #GUPnPContextManager object.
  *
- * Since: 1.1.0
+ * Since: 1.2.0
  **/
 GUPnPContextManager *
 gupnp_context_manager_create_full (GSSDPUDAVersion uda_version, GSocketFamily family, guint port)
@@ -749,7 +737,7 @@ gupnp_context_manager_rescan_control_points (GUPnPContextManager *manager)
  * #GUPnPContextManager::context-available handler after you create a
  * #GUPnPControlPoint object for the newly available context.
  *
- * Since: 0.13.0
+ * Since: 0.14.0
  **/
 void
 gupnp_context_manager_manage_control_point (GUPnPContextManager *manager,
@@ -776,7 +764,7 @@ gupnp_context_manager_manage_control_point (GUPnPContextManager *manager,
  * #GUPnPContextManager::context-available handler after you create a
  * #GUPnPRootDevice object for the newly available context.
  *
- * Since: 0.13.0
+ * Since: 0.14.0
  **/
 void
 gupnp_context_manager_manage_root_device (GUPnPContextManager *manager,
@@ -799,7 +787,7 @@ gupnp_context_manager_manage_root_device (GUPnPContextManager *manager,
  * Get the network port associated with this context manager.
  * Returns: The network port asssociated with this context manager.
  *
- * Since: 0.19.1
+ * Since: 0.20.0
  */
 guint
 gupnp_context_manager_get_port (GUPnPContextManager *manager)
@@ -843,6 +831,7 @@ gupnp_context_manager_get_white_list (GUPnPContextManager *manager)
  * both
  *
  * Returns: The socket family
+ * Since: 1.2.0
  */
 GSocketFamily
 gupnp_context_manager_get_socket_family (GUPnPContextManager *manager)
@@ -864,6 +853,7 @@ gupnp_context_manager_get_socket_family (GUPnPContextManager *manager)
  * Get the UDA protocol version the contexts are implementing
  *
  * Returns: The UDA protocol version
+ * Since: 1.2.0
  */
 GSSDPUDAVersion
 gupnp_context_manager_get_uda_version (GUPnPContextManager *manager)
