@@ -5,20 +5,8 @@
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *         Jorn Baayen <jorn@openedhand.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -67,7 +55,7 @@ gupnp_resource_factory_init (GUPnPResourceFactory *factory)
                                                        NULL);
 }
 
-    static void
+static void
 gupnp_resource_factory_finalize (GObject *object)
 {
     GUPnPResourceFactory *self;
@@ -201,7 +189,7 @@ lookup_type_with_fallback (GHashTable *resource_types,
  * Create a #GUPnPDeviceProxy for the device with element @element, as
  * read from the device description file specified by @location.
  *
- * Return value: A new #GUPnPDeviceProxy.
+ * Return value:(nullable)(transfer full): A new #GUPnPDeviceProxy.
  **/
 GUPnPDeviceProxy *
 gupnp_resource_factory_create_device_proxy
@@ -214,7 +202,7 @@ gupnp_resource_factory_create_device_proxy
                                  const SoupURI        *url_base)
 {
         GUPnPDeviceProxy *proxy;
-        GType             proxy_type = GUPNP_TYPE_DEVICE_PROXY;
+        GType proxy_type;
         GUPnPResourceFactoryPrivate *priv;
 
         g_return_val_if_fail (GUPNP_IS_RESOURCE_FACTORY (factory), NULL);
@@ -253,14 +241,14 @@ gupnp_resource_factory_create_device_proxy
  * @element: The #xmlNode pointing to the right service element
  * @location: The location of the service description file
  * @udn: The UDN of the device the service is contained in
- * @service_type: (allow-none): The service type, or %NULL to use service
+ * @service_type: (nullable): The service type, or %NULL to use service
  * type from @element
  * @url_base: The URL base for this service, or %NULL if none
  *
  * Create a #GUPnPServiceProxy for the service with element @element, as
  * read from the service description file specified by @location.
  *
- * Return value: A new #GUPnPServiceProxy.
+ * Return value:(nullable)(transfer full): A new #GUPnPServiceProxy.
  **/
 GUPnPServiceProxy *
 gupnp_resource_factory_create_service_proxy
@@ -318,7 +306,7 @@ gupnp_resource_factory_create_service_proxy
  * Create a #GUPnPDevice for the device with element @element, as
  * read from the device description file specified by @location.
  *
- * Return value: A new #GUPnPDevice.
+ * Return value: (nullable)(transfer full): A new #GUPnPDevice.
  **/
 GUPnPDevice *
 gupnp_resource_factory_create_device
@@ -374,7 +362,7 @@ gupnp_resource_factory_create_device
  * Create a #GUPnPService for the service with element @element, as
  * read from the service description file specified by @location.
  *
- * Return value: A new #GUPnPService.
+ * Return value: (nullable)(transfer full): A new #GUPnPService.
  **/
 GUPnPService *
 gupnp_resource_factory_create_service
