@@ -3,20 +3,8 @@
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -64,7 +52,7 @@ G_DEFINE_TYPE_EXTENDED (GUPnPRootDevice,
                         G_ADD_PRIVATE(GUPnPRootDevice)
                         G_IMPLEMENT_INTERFACE
                                 (G_TYPE_INITABLE,
-                                 gupnp_root_device_initable_iface_init));
+                                 gupnp_root_device_initable_iface_init))
 
 enum {
         PROP_0,
@@ -273,7 +261,7 @@ fill_resource_group (xmlNode            *element,
         }
 }
 
-/* Load and parse @description_path as an XML document, synchronously. */
+/* Load and parse @description_path as a XML document, synchronously. */
 static GUPnPXMLDoc *
 load_and_parse (const char *description_path)
 {
@@ -484,7 +472,7 @@ gupnp_root_device_class_init (GUPnPRootDeviceClass *klass)
          *
          * Device description document. Constructor property.
          *
-         * Since: 0.13.0
+         * Since: 0.14.0
          **/
         g_object_class_install_property
                 (object_class,
@@ -563,7 +551,7 @@ gupnp_root_device_class_init (GUPnPRootDeviceClass *klass)
  * @description_path: Path to device description document. This could either
  * be an absolute path or path relative to @description_dir.
  * @description_dir: Path to directory where description documents are provided.
- * @error: (allow-none): The location for a #GError to report issue with
+ * @error: (inout)(optional)(nullable): The location for a #GError to report issue with
  * creation on or %NULL.
  *
  * Create a new #GUPnPRootDevice object, automatically loading and parsing
@@ -597,7 +585,7 @@ gupnp_root_device_new (GUPnPContext *context,
  * @description_path: Path to device description document. This could either
  * be an absolute path or path relative to @description_dir.
  * @description_dir: Path to directory where description documents are provided.
- * @error: (allow-none): The location for a #GError to report issue with
+ * @error: (inout)(optional)(nullable): The location for a #GError to report issue with
  * creation on or %NULL.
  *
  * Create a new #GUPnPRootDevice, automatically loading and parsing
@@ -741,7 +729,7 @@ gupnp_root_device_get_description_dir (GUPnPRootDevice *root_device)
  *
  * Returns: (transfer none): The #GSSDPResourceGroup of @root_device.
  *
- * Since: 0.19.2
+ * Since: 0.20.0
  **/
 GSSDPResourceGroup *
 gupnp_root_device_get_ssdp_resource_group (GUPnPRootDevice *root_device)
