@@ -3,20 +3,8 @@
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -67,7 +55,7 @@ typedef struct _GUPnPServiceProxyPrivate GUPnPServiceProxyPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GUPnPServiceProxy,
                             gupnp_service_proxy,
-                            GUPNP_TYPE_SERVICE_INFO);
+                            GUPNP_TYPE_SERVICE_INFO)
 
 enum {
         PROP_0,
@@ -290,10 +278,7 @@ gupnp_service_proxy_dispose (GObject *object)
         }
 
         /* Cancel pending notifications */
-        if (priv->notify_idle_src) {
-                g_source_destroy (priv->notify_idle_src);
-                priv->notify_idle_src = NULL;
-        }
+        g_clear_pointer (&priv->notify_idle_src, g_source_destroy);
 
         g_list_free_full (priv->pending_notifies,
                           (GDestroyNotify) emit_notify_data_free);
@@ -380,18 +365,18 @@ gupnp_service_proxy_class_init (GUPnPServiceProxyClass *klass)
  * gupnp_service_proxy_send_action:
  * @proxy: A #GUPnPServiceProxy
  * @action: An action
- * @error: (allow-none): The location where to store any error, or %NULL
+ * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
  * @...: tuples of in parameter name, in parameter type, and in parameter
  * value, followed by %NULL, and then tuples of out parameter name,
  * out parameter type, and out parameter value location, terminated with %NULL
  *
  * Sends action @action with parameters @Varargs to the service exposed by
  * @proxy synchronously. If an error occurred, @error will be set. In case of
- * a UPnPError the error code will be the same in @error.
+ * an UPnPError the error code will be the same in @error.
  *
  * Return value: %TRUE if sending the action was succesful.
  *
- * Deprecated: 1.1.2: Use gupnp_service_proxy_action_new() and
+ * Deprecated: 1.2.0: Use gupnp_service_proxy_action_new() and
  * gupnp_service_proxy_call_action()
  **/
 gboolean
@@ -419,7 +404,7 @@ gupnp_service_proxy_send_action (GUPnPServiceProxy *proxy,
  * gupnp_service_proxy_send_action_valist:
  * @proxy: A #GUPnPServiceProxy
  * @action: An action
- * @error: (allow-none): The location where to store any error, or %NULL
+ * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
  * @var_args: va_list of tuples of in parameter name, in parameter type, and in
  * parameter value, followed by %NULL, and then tuples of out parameter name,
  * out parameter type, and out parameter value location
@@ -428,7 +413,7 @@ gupnp_service_proxy_send_action (GUPnPServiceProxy *proxy,
  *
  * Return value: %TRUE if sending the action was succesful.
  *
- * Deprecated: 1.1.2
+ * Deprecated: 1.2.0
  **/
 gboolean
 gupnp_service_proxy_send_action_valist (GUPnPServiceProxy *proxy,
@@ -457,7 +442,7 @@ gupnp_service_proxy_send_action_valist (GUPnPServiceProxy *proxy,
 
         handle = gupnp_service_proxy_action_new_from_list (action_name, in_names, in_values);
 
-        if (gupnp_service_proxy_call_action (proxy, handle, NULL,  error) == NULL) {
+        if (gupnp_service_proxy_call_action (proxy, handle, NULL, error) == NULL) {
                 result = FALSE;
                 goto out;
         }
@@ -483,7 +468,7 @@ out:
 
 /**
  * gupnp_service_proxy_send_action_list:
- * @proxy: (transfer none) : A #GUPnPServiceProxy
+ * @proxy: (transfer none): A #GUPnPServiceProxy
  * @action: An action
  * @in_names: (element-type utf8) (transfer none): #GList of 'in' parameter
  * names (as strings)
@@ -495,14 +480,14 @@ out:
  * that line up with @out_names
  * @out_values: (element-type GValue) (transfer full) (out): #GList of values
  * (as #GValue) that line up with @out_names and @out_types.
- * @error: (allow-none): The location where to store any error, or %NULL
+ * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
  *
  * The synchronous variant of #gupnp_service_proxy_begin_action_list and
  * #gupnp_service_proxy_end_action_list.
  *
  * Return value: %TRUE if sending the action was succesful.
  *
- * Deprecated: 1.1.2: Use gupnp_service_proxy_action_new_from_list() and gupnp_service_proxy_call_action()
+ * Deprecated: 1.2.0: Use gupnp_service_proxy_action_new_from_list() and gupnp_service_proxy_call_action()
  *
  **/
 gboolean
@@ -551,8 +536,13 @@ on_legacy_async_callback (GObject *source, GAsyncResult *res, gpointer user_data
 
         /* Do not perform legacy call-back if action is cancelled, to comply with the old implementation */
         if (action->callback != NULL &&
-            !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+            !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                if (error != NULL)
+                        g_propagate_error (&action->error, error);
                 action->callback (action->proxy, action, action->user_data);
+        }
+
+        g_clear_error (&error);
 }
 
 /**
@@ -574,8 +564,8 @@ on_legacy_async_callback (GObject *source, GAsyncResult *res, gpointer user_data
  * gupnp_service_proxy_cancel_action() or
  * gupnp_service_proxy_end_action_valist().
  *
- * Deprecated: 1.1.2: Use gupnp_service_proxy_action_new() and
- * gupnp_service_proxy_action_call_action_async()
+ * Deprecated: 1.2.0: Use gupnp_service_proxy_action_new() and
+ * gupnp_service_proxy_call_action_async()
  **/
 GUPnPServiceProxyAction *
 gupnp_service_proxy_begin_action (GUPnPServiceProxy              *proxy,
@@ -617,7 +607,7 @@ on_action_cancelled (GCancellable *cancellable, gpointer user_data)
         GUPnPContext *context;
         SoupSession *session;
 
-        if (action->msg != NULL) {
+        if (action->msg != NULL && action->proxy != NULL) {
                 context = gupnp_service_info_get_context
                                         (GUPNP_SERVICE_INFO (action->proxy));
                 session = gupnp_context_get_session (context);
@@ -629,103 +619,81 @@ on_action_cancelled (GCancellable *cancellable, gpointer user_data)
         }
 }
 
+
+
 /* Begins a basic action message */
-static void
-prepare_action_msg (GUPnPServiceProxy              *proxy,
-                    GUPnPServiceProxyAction        *ret,
-                    GCancellable                   *cancellable)
+static gboolean
+prepare_action_msg (GUPnPServiceProxy *proxy,
+                    GUPnPServiceProxyAction *action,
+                    GError **error)
 {
-        GUPnPServiceProxyPrivate *priv;
         char *control_url, *full_action;
         const char *service_type;
 
-        priv = gupnp_service_proxy_get_instance_private (proxy);
-
-        ret->proxy = proxy;
-        g_object_add_weak_pointer (G_OBJECT (proxy), (gpointer *)&(ret->proxy));
-
-        priv->pending_actions = g_list_prepend (priv->pending_actions, ret);
-
-        if (cancellable != NULL) {
-                ret->cancellable = g_object_ref (cancellable);
-        } else {
-                ret->cancellable = g_cancellable_new ();
-        }
-        ret->cancellable_connection_id = g_cancellable_connect (ret->cancellable,
-                                                                G_CALLBACK (on_action_cancelled),
-                                                                ret,
-                                                                NULL);
-
         /* Make sure we have a service type */
         service_type = gupnp_service_info_get_service_type
                                         (GUPNP_SERVICE_INFO (proxy));
         if (service_type == NULL) {
-                ret->error = g_error_new (GUPNP_SERVER_ERROR,
-                                          GUPNP_SERVER_ERROR_OTHER,
-                                          "No service type defined");
+                g_propagate_error (error,
+                                   g_error_new (GUPNP_SERVER_ERROR,
+                                                GUPNP_SERVER_ERROR_OTHER,
+                                                "No service type defined"));
 
-                return;
+                return FALSE;
         }
 
         /* Create message */
         control_url = gupnp_service_info_get_control_url
                                         (GUPNP_SERVICE_INFO (proxy));
 
-        if (control_url != NULL) {
-                GUPnPContext *context = NULL;
-                char *local_control_url = NULL;
+        if (control_url == NULL) {
+                g_propagate_error (
+                        error,
+                        g_error_new (GUPNP_SERVER_ERROR,
+                                     GUPNP_SERVER_ERROR_INVALID_URL,
+                                     "No valid control URL defined"));
 
-                context = gupnp_service_info_get_context
-                                        (GUPNP_SERVICE_INFO (proxy));
-
-                local_control_url = gupnp_context_rewrite_uri (context,
-                                                               control_url);
-                g_free (control_url);
-
-                ret->msg = soup_message_new (SOUP_METHOD_POST, local_control_url);
-                g_free (local_control_url);
+                return FALSE;
         }
 
-        if (ret->msg == NULL) {
-                ret->error = g_error_new (GUPNP_SERVER_ERROR,
-                                          GUPNP_SERVER_ERROR_INVALID_URL,
-                                          "No valid control URL defined");
+        GUPnPContext *context = NULL;
+        char *local_control_url = NULL;
 
-                return;
-        }
+        context = gupnp_service_info_get_context (GUPNP_SERVICE_INFO (proxy));
+
+        local_control_url = gupnp_context_rewrite_uri (context, control_url);
+        g_free (control_url);
+
+        g_clear_object (&action->msg);
+        action->msg = soup_message_new (SOUP_METHOD_POST, local_control_url);
+        g_free (local_control_url);
 
         /* Specify action */
-        full_action = g_strdup_printf ("\"%s#%s\"", service_type, ret->name);
-        soup_message_headers_append (ret->msg->request_headers,
+        full_action = g_strdup_printf ("\"%s#%s\"", service_type, action->name);
+        soup_message_headers_append (action->msg->request_headers,
                                      "SOAPAction",
                                      full_action);
         g_free (full_action);
 
         /* Specify language */
-        http_request_set_accept_language (ret->msg);
+        http_request_set_accept_language (action->msg);
 
         /* Accept gzip encoding */
-        soup_message_headers_append (ret->msg->request_headers,
+        soup_message_headers_append (action->msg->request_headers,
                                      "Accept-Encoding", "gzip");
 
-        g_string_insert (ret->msg_str, ret->header_pos, "<u:");
-        ret->header_pos += strlen("<u:");
-        g_string_insert (ret->msg_str, ret->header_pos, ret->name);
-        ret->header_pos += strlen (ret->name);
-        g_string_insert (ret->msg_str, ret->header_pos, " xmlns:u=\"");
-        ret->header_pos += strlen(" xmlns:u=\"");
-        g_string_insert (ret->msg_str, ret->header_pos, service_type);
-        ret->header_pos += strlen (service_type);
-        g_string_insert (ret->msg_str, ret->header_pos, "\">");
+        gupnp_service_proxy_action_serialize (action, service_type);
 
-        soup_message_set_request (ret->msg,
+        soup_message_set_request (action->msg,
                                   "text/xml; charset=\"utf-8\"",
                                   SOUP_MEMORY_TAKE,
-                                  ret->msg_str->str,
-                                  ret->msg_str->len);
+                                  action->msg_str->str,
+                                  action->msg_str->len);
 
-        g_string_free (ret->msg_str, FALSE);
-        ret->msg_str = NULL;
+        g_string_free (action->msg_str, FALSE);
+        action->msg_str = NULL;
+
+        return TRUE;
 }
 
 static void
@@ -819,6 +787,7 @@ gupnp_service_proxy_action_queue_task (GTask *task)
                                     action->msg,
                                     (SoupSessionCallback) action_task_got_response,
                                     task);
+        action->pending = TRUE;
 }
 
 /**
@@ -837,7 +806,7 @@ gupnp_service_proxy_action_queue_task (GTask *task)
  * be freed when calling gupnp_service_proxy_cancel_action() or
  * gupnp_service_proxy_end_action_valist().
  *
- * Deprecated: 1.1.2
+ * Deprecated: 1.2.0
  **/
 GUPnPServiceProxyAction *
 gupnp_service_proxy_begin_action_valist
@@ -889,7 +858,7 @@ gupnp_service_proxy_begin_action_valist
  * be freed when calling gupnp_service_proxy_cancel_action() or
  * gupnp_service_proxy_end_action_list().
  *
- * Since: 0.13.3
+ * Since: 0.14.0
  **/
 GUPnPServiceProxyAction *
 gupnp_service_proxy_begin_action_list
@@ -920,7 +889,7 @@ gupnp_service_proxy_begin_action_list
  * gupnp_service_proxy_end_action:
  * @proxy: A #GUPnPServiceProxy
  * @action: A #GUPnPServiceProxyAction handle
- * @error: (allow-none): The location where to store any error, or %NULL
+ * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
  * @...: tuples of out parameter name, out parameter type, and out parameter
  * value location, terminated with %NULL. The out parameter values should be
  * freed after use
@@ -965,7 +934,7 @@ gupnp_service_proxy_end_action (GUPnPServiceProxy       *proxy,
  * gupnp_service_proxy_end_action_valist:
  * @proxy: A #GUPnPServiceProxy
  * @action: A #GUPnPServiceProxyAction handle
- * @error: (allow-none): The location where to store any error, or %NULL
+ * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
  * @var_args: A va_list of tuples of out parameter name, out parameter type,
  * and out parameter value location. The out parameter values should be
  * freed after use
@@ -1005,7 +974,7 @@ gupnp_service_proxy_end_action_valist (GUPnPServiceProxy       *proxy,
  * gupnp_service_proxy_end_action_list:
  * @proxy: A #GUPnPServiceProxy
  * @action: A #GUPnPServiceProxyAction handle
- * @error: The location where to store any error, or %NULL
+ * @error:(inout)(optional)(nullable): The location where to store any error, or %NULL
  * @out_names: (element-type utf8) (transfer none): #GList of 'out' parameter
  * names (as strings)
  * @out_types: (element-type GType) (transfer none): #GList of types (as #GType)
@@ -1057,7 +1026,7 @@ gupnp_service_proxy_end_action_list (GUPnPServiceProxy       *proxy,
  * gupnp_service_proxy_end_action_hash:
  * @proxy: A #GUPnPServiceProxy
  * @action: A #GUPnPServiceProxyAction handle
- * @error: The location where to store any error, or %NULL
+ * @error:(inout)(optional)(nullable): The location where to store any error, or %NULL
  * @hash: (element-type utf8 GValue) (inout) (transfer none): A #GHashTable of
  * out parameter name and initialised #GValue pairs
  *
@@ -1104,7 +1073,7 @@ gupnp_service_proxy_end_action_hash
  *
  * Cancels @action, freeing the @action handle.
  *
- * Deprecated: 1.1.2: Use the #GCancellable passed to
+ * Deprecated: 1.2.0: Use the #GCancellable passed to
  * gupnp_service_proxy_call_action_async() or gupnp_service_proxy_call_action()
  **/
 void
@@ -2017,10 +1986,10 @@ unsubscribe (GUPnPServiceProxy *proxy)
  *
  * (Un)subscribes to this service.
  *
- * Note that the relevant messages are not immediately sent but queued.
+ * <note>The relevant messages are not immediately sent but queued.
  * If you want to unsubcribe from this service because the application
  * is quitting, rely on automatic synchronised unsubscription on object
- * destruction instead.
+ * destruction instead.</note>
  **/
 void
 gupnp_service_proxy_set_subscribed (GUPnPServiceProxy *proxy,
@@ -2080,6 +2049,7 @@ gupnp_service_proxy_get_subscribed (GUPnPServiceProxy *proxy)
  * gupnp_service_proxy_action_get_result_hash() or
  * gupnp_service_proxy_action_get_result_list() to extract the result of the
  * remote call.
+ * Since: 1.2.0
  */
 void
 gupnp_service_proxy_call_action_async (GUPnPServiceProxy       *proxy,
@@ -2089,20 +2059,41 @@ gupnp_service_proxy_call_action_async (GUPnPServiceProxy       *proxy,
                                        gpointer                user_data)
 {
         GTask *task;
+        GUPnPServiceProxyPrivate *priv;
+        GError *error = NULL;
 
         g_return_if_fail (GUPNP_IS_SERVICE_PROXY (proxy));
 
+        priv = gupnp_service_proxy_get_instance_private (proxy);
         task = g_task_new (proxy, cancellable, callback, user_data);
+        char *task_name = g_strdup_printf ("UPnP Call \"%s\"", action->name);
+        g_task_set_name (task, task_name);
+        g_free (task_name);
         g_task_set_task_data (task,
                               gupnp_service_proxy_action_ref (action),
                               (GDestroyNotify) gupnp_service_proxy_action_unref);
 
-        prepare_action_msg (proxy, action, cancellable);
+        prepare_action_msg (proxy, action, &error);
 
-        if (action->error != NULL) {
-                g_task_return_error (task, g_error_copy (action->error));
+        if (error != NULL) {
+                g_task_return_error (task, error);
                 g_object_unref (task);
         } else {
+                action->proxy = proxy;
+                g_object_add_weak_pointer (G_OBJECT (proxy), (gpointer *)&(action->proxy));
+
+                priv->pending_actions = g_list_prepend (priv->pending_actions, action);
+
+                if (cancellable != NULL) {
+                        action->cancellable = g_object_ref (cancellable);
+                } else {
+                        action->cancellable = g_cancellable_new ();
+                }
+                action->cancellable_connection_id = g_cancellable_connect (action->cancellable,
+                                                                           G_CALLBACK (on_action_cancelled),
+                                                                           action,
+                                                                           NULL);
+
                 gupnp_service_proxy_action_queue_task (task);
         }
 }
@@ -2111,12 +2102,13 @@ gupnp_service_proxy_call_action_async (GUPnPServiceProxy       *proxy,
  * gupnp_service_proxy_call_action_finish:
  * @proxy: a #GUPnPServiceProxy
  * @result: a #GAsyncResult
- * @error: (allow-none): Return location for a #GError, or %NULL
+ * @error: (inout)(optional)(nullable): Return location for a #GError, or %NULL
  *
  * Finish an asynchronous call initiated with
  * gupnp_service_proxy_call_action_async().
  *
  * Returns: (nullable) (transfer none): %NULL, if the call had an error, the action otherwise.
+ * Since: 1.2.0
  */
 GUPnPServiceProxyAction *
 gupnp_service_proxy_call_action_finish (GUPnPServiceProxy *proxy,
@@ -2125,7 +2117,12 @@ gupnp_service_proxy_call_action_finish (GUPnPServiceProxy *proxy,
 {
         g_return_val_if_fail (g_task_is_valid (G_TASK (result), proxy), NULL);
 
-        return g_task_propagate_pointer (G_TASK (result), error);
+        GUPnPServiceProxyAction *action =
+                g_task_propagate_pointer (G_TASK (result), error);
+        g_clear_weak_pointer (&action->proxy);
+        action->pending = FALSE;
+
+        return action;
 }
 
 /**
@@ -2134,11 +2131,12 @@ gupnp_service_proxy_call_action_finish (GUPnPServiceProxy *proxy,
  * @action: (transfer none): An action
  * @cancellable: (allow-none): A #GCancellable which can be used to cancel the
  * current action call
- * @error: (allow-none): Return location for a #GError, or %NULL.
+ * @error: (inout)(optional)(nullable): Return location for a #GError, or %NULL.
  *
  * Synchronously call the @action on the remote UPnP service.
  *
  * Returns: (nullable)(transfer none): %NULL on error, @action if successful.
+ * Since: 1.2.0
  */
 GUPnPServiceProxyAction *
 gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
@@ -2150,22 +2148,31 @@ gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
         SoupSession *session = NULL;
 
         g_return_val_if_fail (GUPNP_IS_SERVICE_PROXY (proxy), NULL);
+        g_return_val_if_fail (!action->pending, NULL);
 
-        prepare_action_msg (proxy, action, cancellable);
-
-        /* prepare_action_msg has queued the message, so remove it */
-        gupnp_service_proxy_remove_action (proxy, action);
-
-        if (action->error != NULL) {
-                g_propagate_error (error, g_error_copy (action->error));
-
+        if (!prepare_action_msg (proxy, action, error)) {
                 return NULL;
         }
 
+        if (cancellable != NULL) {
+                action->cancellable = g_object_ref (cancellable);
+        } else {
+                action->cancellable = g_cancellable_new ();
+        }
+        action->cancellable_connection_id = g_cancellable_connect (action->cancellable,
+                                                                   G_CALLBACK (on_action_cancelled),
+                                                                   action,
+                                                                   NULL);
+
         context = gupnp_service_info_get_context (GUPNP_SERVICE_INFO (proxy));
         session = gupnp_context_get_session (context);
         soup_session_send_message (session, action->msg);
 
+        g_cancellable_disconnect (action->cancellable,
+                                  action->cancellable_connection_id);
+        action->cancellable_connection_id = 0;
+        g_clear_object (&action->cancellable);
+
         /* If not allowed, try again */
         if (action->msg->status_code == SOUP_STATUS_METHOD_NOT_ALLOWED) {
                 update_message_after_not_allowed (action->msg);
@@ -2173,10 +2180,11 @@ gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
         }
 
         if (action->msg->status_code == SOUP_STATUS_CANCELLED) {
-                action->error = g_error_new (G_IO_ERROR,
-                                             G_IO_ERROR_CANCELLED,
-                                             "Action message was cancelled");
-                g_propagate_error (error, g_error_copy (action->error));
+                g_propagate_error (
+                        error,
+                        g_error_new (G_IO_ERROR,
+                                     G_IO_ERROR_CANCELLED,
+                                     "Action message was cancelled"));
 
                 return NULL;
         }
