@@ -3,20 +3,8 @@
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -74,7 +62,7 @@ typedef struct _GUPnPServicePrivate GUPnPServicePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GUPnPService,
                             gupnp_service,
-                            GUPNP_TYPE_SERVICE_INFO);
+                            GUPNP_TYPE_SERVICE_INFO)
 
 enum {
         PROP_0,
@@ -436,7 +424,7 @@ gupnp_service_action_get_valist (GUPnPServiceAction *action,
  * #GValue. g_list_free() the returned list and g_value_unset() and g_slice_free()
  * each element.
  *
- * Since: 0.13.3
+ * Since: 0.14.0
  **/
 GList *
 gupnp_service_action_get_values (GUPnPServiceAction *action,
@@ -519,7 +507,7 @@ gupnp_service_action_get_value (GUPnPServiceAction *action,
  * Return value: (transfer full): Value as #GValue associated with @action.
  * g_value_unset() and g_slice_free() it after usage.
  *
- * Since: 0.13.3
+ * Since: 0.14.0
  **/
 GValue *
 gupnp_service_action_get_gvalue (GUPnPServiceAction *action,
@@ -544,7 +532,7 @@ gupnp_service_action_get_gvalue (GUPnPServiceAction *action,
  *
  * Return value: The number of IN arguments from the @action.
  *
- * Since: 0.17.0
+ * Since: 0.18.0
  */
 guint
 gupnp_service_action_get_argument_count (GUPnPServiceAction *action)
@@ -624,11 +612,11 @@ gupnp_service_action_set_valist (GUPnPServiceAction *action,
  * @action: A #GUPnPServiceAction
  * @arg_names: (element-type utf8) (transfer none): A #GList of argument names
  * @arg_values: (element-type GValue) (transfer none): The #GList of values (as
- * #GValues) that line up with @arg_names.
+ * #GValue<!-- -->s) that line up with @arg_names.
  *
  * Sets the specified action return values.
  *
- * Since: 0.13.3
+ * Since: 0.14.0
  **/
 void
 gupnp_service_action_set_values (GUPnPServiceAction *action,
@@ -806,7 +794,7 @@ gupnp_service_action_return_error (GUPnPServiceAction *action,
  * Return value: (transfer full): #SoupMessage associated with @action. Unref
  * after using it.
  *
- * Since: 0.13.0
+ * Since: 0.14.0
  **/
 SoupMessage *
 gupnp_service_action_get_message (GUPnPServiceAction *action)
@@ -1208,29 +1196,22 @@ add_subscription_callback (GUPnPContext *context,
                            const char *callback)
 {
             SoupURI *local_uri = NULL;
-            char *host = NULL;
-            char *index = NULL;
 
             local_uri = gupnp_context_rewrite_uri_to_uri (context, callback);
             if (local_uri == NULL) {
                     return list;
             }
 
+            const char *host = soup_uri_get_host (local_uri);
+            GSocketAddress *address = g_inet_socket_address_new_from_string (host, 0);
 
-            host = g_strdup (soup_uri_get_host (local_uri));
-            index = g_strrstr(host, "%");
-            // Cut off network index
-            if (index != NULL) {
-                    *index = '\0';
-            }
             // CVE-2020-12695: Ignore subscription call-backs that are not "in
             // our network segment"
-            if (gupnp_context_ip_is_ours (context, host)) {
+            if (gssdp_client_can_reach (GSSDP_CLIENT (context), G_INET_SOCKET_ADDRESS (address))) {
                     list = g_list_append (list, local_uri);
             } else {
                     g_warning ("%s is not in our network; ignoring", callback);
             }
-            g_free (host);
 
             return list;
 }
@@ -1398,6 +1379,20 @@ subscription_server_handler (G_GNUC_UNUSED SoupServer        *server,
 
         service = GUPNP_SERVICE (user_data);
 
+        const char *host =
+                soup_message_headers_get_one (msg->request_headers, "Host");
+        GUPnPContext *context = gupnp_service_info_get_context (user_data);
+        if (!gupnp_context_validate_host_header(context, host)) {
+                g_warning ("Host header mismatch, expected %s:%d, got %s",
+                           gssdp_client_get_host_ip (GSSDP_CLIENT (context)),
+                           gupnp_context_get_port (context),
+                           host);
+
+                soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
+
+                return;
+        }
+
         callback = soup_message_headers_get_one (msg->request_headers,
                                                  "Callback");
         nt       = soup_message_headers_get_one (msg->request_headers, "NT");
@@ -1736,7 +1731,6 @@ gupnp_service_finalize (GObject *object)
         GUPnPService *service;
         GUPnPServicePrivate *priv;
         GObjectClass *object_class;
-        NotifyData *data;
 
         service = GUPNP_SERVICE (object);
         priv = gupnp_service_get_instance_private (service);
@@ -1748,20 +1742,11 @@ gupnp_service_finalize (GObject *object)
         g_list_free_full (priv->state_variables, g_free);
 
         /* Free notify queue */
-        while ((data = g_queue_pop_head (priv->notify_queue)))
-                notify_data_free (data);
+        g_queue_free_full (priv->notify_queue,
+                           (GDestroyNotify) notify_data_free);
 
-        g_queue_free (priv->notify_queue);
-
-        if (priv->session) {
-                g_object_unref (priv->session);
-                priv->session = NULL;
-        }
-
-        if (priv->introspection) {
-                g_object_unref (priv->introspection);
-                priv->introspection = NULL;
-        }
+        g_clear_object (&priv->session);
+        g_clear_object (&priv->introspection);
 
         /* Call super */
         object_class = G_OBJECT_CLASS (gupnp_service_parent_class);
@@ -2003,27 +1988,27 @@ notify_got_response (G_GNUC_UNUSED SoupSession *session,
 }
 
 /* Send notification @user_data to subscriber @value */
-    static void
-    notify_subscriber (G_GNUC_UNUSED gpointer key,
-                       gpointer               value,
-                       gpointer               user_data)
-    {
-            SubscriptionData *data;
-            const char *property_set;
-            char *tmp;
-            SoupMessage *msg;
-            SoupSession *session;
+static void
+notify_subscriber (G_GNUC_UNUSED gpointer key,
+                   gpointer value,
+                   gpointer user_data)
+{
+        SubscriptionData *data;
+        const char *property_set;
+        char *tmp;
+        SoupMessage *msg;
+        SoupSession *session;
 
-            data = value;
-            property_set = user_data;
+        data = value;
+        property_set = user_data;
 
-            /* Subscriber called unsubscribe */
-            if (subscription_data_can_delete (data))
-                    return;
+        /* Subscriber called unsubscribe */
+        if (subscription_data_can_delete (data))
+                return;
 
-            /* Create message */
-            msg = soup_message_new_from_uri (GENA_METHOD_NOTIFY,
-                                             data->callbacks->data);
+        /* Create message */
+        msg = soup_message_new_from_uri (GENA_METHOD_NOTIFY,
+                                         data->callbacks->data);
 
         soup_message_headers_append (msg->request_headers,
                                      "NT",
@@ -2325,7 +2310,7 @@ connect_names_to_signal_handlers (GUPnPService *service,
  * gupnp_service_signals_autoconnect:
  * @service: A #GUPnPService
  * @user_data: the data to pass to each of the callbacks
- * @error: return location for a #GError, or %NULL
+ * @error: (inout)(optional)(nullable): return location for a #GError, or %NULL
  *
  * A convenience function that attempts to connect all possible
  * #GUPnPService::action-invoked and #GUPnPService::query-variable signals to
