@@ -3,20 +3,8 @@
  *
  * Author: Ludovic Ferrandis <ludovic.ferrandis@intel.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -44,7 +32,7 @@ typedef struct _GUPnPWhiteListPrivate GUPnPWhiteListPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GUPnPWhiteList,
                             gupnp_white_list,
-                            G_TYPE_OBJECT);
+                            G_TYPE_OBJECT)
 
 enum {
         PROP_0,
@@ -368,7 +356,7 @@ gupnp_white_list_remove_entry (GUPnPWhiteList *white_list, const gchar* entry)
  *
  * Get the #GList of entries that compose the white list. Do not free
  *
- * Return value: (element-type utf8) (transfer none):  a #GList of entries
+ * Return value: (nullable)(element-type utf8) (transfer none):  a #GList of entries
  * used to filter networks, interfaces,... or %NULL.
  * Do not modify or free the list nor its elements.
  *
